package com.biganto.visual.roompark.domain.model

import androidx.annotation.StringRes
import com.biganto.visual.roompark.R

data class DealPreviewModel(
    val id:String,
    val type:String,
    val name:String,
    val building:Int?,
    val section:Int?,
    val floor:Int?,
    val area: Float?,
    val statusNo : Int,
    val dealSum : Int,
    val dealPayout : Int,
    val dealToPay : Int,
    val dealTourIds : List<Int?>?,
    val isViewed: Boolean = true,
    val statusList : List<StatusModel>
){
    constructor(data:Pair<DealModel,List<StatusModel>>) : this(
        data.first.id,
        data.first.estate.type,
        data.first.estate.number,
        data.first.estate.commonInfo?.building,
        data.first.estate.commonInfo?.section_begin,
        data.first.estate.commonInfo?.floor,
        data.first.estate.commonInfo?.area,
        data.first.statusId,
        data.first.opportunitySum,
        data.first.paymentSum,
        data.first.amount_pay_sum,
        arrayListOf<Int?>(data.first.estate.multitourId),
        false,
        data.second
    )
}



@StringRes
fun DealPreviewModel.typeDoubleString() =
    when(this.type){
        "flat" -> R.string.estate_type_flat_long
        "parking" -> R.string.estate_type_parking_long
        "storage" -> R.string.estate_type_store_long
        else -> R.string.estate_type_other_long
    }

@StringRes
fun DealPreviewModel.typeShortString() =
    when(this.type){
        "flat" -> R.string.estate_type_flat_short
        "parking" -> R.string.estate_type_parking_short
        "storage" -> R.string.estate_type_store_short
        else -> R.string.estate_type_other_short
    }