package com.biganto.visual.roompark.presentation.screen.article

import PicassoImageGetter
import android.os.Build
import android.os.Bundle
import android.text.Html
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.core.os.bundleOf
import androidx.core.text.HtmlCompat
import butterknife.BindView
import butterknife.OnClick
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.util.extensions.formatToSimple
import com.google.android.material.floatingactionbutton.FloatingActionButton
import com.google.android.material.textview.MaterialTextView
import com.squareup.picasso.Picasso
import jp.wasabeef.picasso.transformations.BlurTransformation
import timber.log.Timber
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

private const val ARTICLE_ID = "DIRECT_ARTICLE_ID_KEY"

class ArticleScreenController :
    BigantoBaseController<ArticleScreenViewState
            , ArticleScreen
            , ArticleScreenPresenter>
    , ArticleScreen {

    constructor(args:Bundle):super(args)

    constructor(id: Int) : super(bundleOf(ARTICLE_ID to id))

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: ArticleScreenPresenter

    @BindView(R.id.articleTitle)
    lateinit var title:MaterialTextView

    @BindView(R.id.articleHeaderBlock)
    lateinit var headerBlock: ViewGroup

    @BindView(R.id.articleContent)
    lateinit var contentView: MaterialTextView

    @BindView(R.id.articleCloseButton)
    lateinit var closeArticle: FloatingActionButton


    val blurPreview:ImageView by lazy {
       headerBlock.findViewById<ImageView>(R.id.articlePreviewBlurred)
    }

    val articlePreview:ImageView by lazy {
        headerBlock.findViewById<ImageView>(R.id.articlePreview)
    }

    val articleDate:MaterialTextView by lazy {
        headerBlock.findViewById<MaterialTextView>(R.id.articleDate)
    }

    val closeButton:FloatingActionButton by lazy {
        headerBlock.findViewById<FloatingActionButton>(R.id.articleCloseButton)
    }

    @OnClick(R.id.articleCloseButton)
    fun onCloseArticle(){
        handleBack()
    }



    private fun setToolbar(){
        toolBar.hideAll()
        toolBar.appBar.setExpanded(false,false)
//        toolBar.collapsingToolbarLayout.stateTitle = "ИЗБРАННОЕ"
//        toolBar.appBar.liftOnScrollTargetViewId = R.id.Article_cards_recycler_view
        toolBar.appBar.setLiftable(false)
        toolBar.appBarScrollable(false)
//        ArticleRecyclerView.isNestedScrollingEnabled = false
    }

    private fun bindRecycler() {

    }

    override fun onViewBound(v: View) {
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: ArticleScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is ArticleScreenViewState.Idle -> render(viewState)
            is ArticleScreenViewState.ArticleLoaded -> render(viewState)
        }
    }

    private fun render(viewState: ArticleScreenViewState.Idle){

    }


    private fun render(viewState: ArticleScreenViewState.ArticleLoaded) {

        title.text = viewState.item.title

        val imageGetter = PicassoImageGetter(contentView)
        val parsedHtml = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N)
            Html.fromHtml(viewState.item.htmlBody.replace("<br>","<br />"),
                HtmlCompat.FROM_HTML_MODE_COMPACT
                ,
                imageGetter
//                Html.ImageGetter { url ->
//                    val rUrl = "room-park.ru$url"
//                    Timber.d("trying to get image: $rUrl")
//                    Picasso.get()
//                        .load(rUrl)
//                        .get()
//                        .toDrawable(resources!!)
////                        .current
//                }
                , null)
        else Html.fromHtml(viewState.item.htmlBody.replace("<br>","<br />")
            ,
            imageGetter
//            Html.ImageGetter { url ->
//            val rUrl = "room-park.ru$url"
//            Timber.d("trying to get image: $rUrl")
//                Picasso.get()
//                    .load(rUrl)
//                    .get()
//                    .toDrawable(resources!!)
////                    .current
//            }
            , null)

        contentView.text = parsedHtml

        articleDate.text = viewState.item.published.formatToSimple

        Picasso.get()
            .load(viewState.item.previewUrl)
            .transform(BlurTransformation(activity,40,4))
            .into(blurPreview)

        Picasso.get()
            .load(viewState.item.previewUrl)
            .into(articlePreview)
    }

    private fun getComponent() = DaggerArticleScreenComponent.factory()
        .create(RoomParkApplication.component
            ,activity as RoomParkMainActivity
            ,args.getInt(ARTICLE_ID))
        .inject(this)

    override fun getLayoutId(): Int = R.layout.feed_read_screen


}

