package com.biganto.visual.roompark.presentation.screen.deals

import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.HeaderToolbarModel
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.deal.DealScreenController
import com.biganto.visual.roompark.presentation.screen.deals.util.DealsListAdapter
import com.biganto.visual.roompark.util.view_utils.grid.CeilsDecoration
import com.bluelinelabs.conductor.RouterTransaction
import com.bluelinelabs.conductor.changehandler.FadeChangeHandler
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class DealsScreenController  :
    BigantoBaseController<DealsScreenViewState
        , DealsScreen
        , DealsScreenPresenter>()
    , DealsScreen {

    override fun injectDependencies() {
        getComponent()
    }

    @BindView(R.id.favorites_cards_recycler_view)
    lateinit var dealsRecyclerView: RecyclerView

    private fun bindRecycler() {
        dealsRecyclerView.isNestedScrollingEnabled = true
        dealsRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        dealsRecyclerView.adapter = DealsListAdapter()
        dealsRecyclerView.itemAnimator = null

        if (dealsRecyclerView.itemDecorationCount == 0)
            dealsRecyclerView.addItemDecoration(
                CeilsDecoration(1
                    , resources?.getDimensionPixelSize(R.dimen.ceil_grid_padding))
            )
    }

    private fun setToolbar(){

        toolBar.showAll()
        toolBar.appBar.setExpanded(false,false)
        toolBar.appBar.liftOnScrollTargetViewId = R.id.favorites_cards_recycler_view
        toolBar.appBar.setLiftable(true)
        toolBar.appBarScrollable(false)
        dealsRecyclerView.isNestedScrollingEnabled = false

        toolBar.setToolbar(
            HeaderToolbarModel(
                false,null
                ,resources?.getString(R.string.my_deals),null)
        )

    }

    @Inject
    override lateinit var injectedPresenter: DealsScreenPresenter


    override fun onViewBound(v: View) {
        setToolbar()
        bindRecycler()
        detachDisposable.addAll(
            (dealsRecyclerView.adapter as DealsListAdapter)
                .onItemClicked
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    Timber.d("got card clicked $it")
                    router.pushController(RouterTransaction.with(DealScreenController(it.id))
                        .pushChangeHandler(FadeChangeHandler())
                        .popChangeHandler(FadeChangeHandler())
                    )
                }
        )
    }

    override fun render(viewState: DealsScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is DealsScreenViewState.Idle -> render(viewState)
            is DealsScreenViewState.DealsLoaded -> render(viewState)
            is DealsScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: DealsScreenViewState.Idle){

    }

    private fun render(viewState: DealsScreenViewState.DealsLoaded){
        (dealsRecyclerView.adapter as DealsListAdapter).addItems(viewState.items)
    }

    private fun render(viewState: DealsScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun getComponent() = DaggerDealsScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.favorites_screen


}