package com.biganto.visual.roompark.presentation.screen.favorites

import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.HeaderToolbarModel
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.estate.EstateScreenController
import com.biganto.visual.roompark.presentation.screen.favorites.util.FavoritesListAdapter
import com.biganto.visual.roompark.util.view_utils.grid.CeilsDecoration
import com.bluelinelabs.conductor.RouterTransaction
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class FavoritesScreenController  :
    BigantoBaseController<FavoritesScreenViewState
        , FavoritesScreen
        , FavoritesScreenPresenter>()
    , FavoritesScreen {

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: FavoritesScreenPresenter

    @BindView(R.id.favorites_cards_recycler_view)
    lateinit var favoritesRecyclerView: RecyclerView



    private fun setToolbar(){
        toolBar.showAll()
        toolBar.appBar.setExpanded(false,false)
        toolBar.appBar.liftOnScrollTargetViewId = R.id.favorites_cards_recycler_view
        toolBar.appBar.setLiftable(true)
        toolBar.appBarScrollable(false)
        favoritesRecyclerView.isNestedScrollingEnabled = false

        toolBar.setToolbar(
            HeaderToolbarModel(
                false,null
                ,resources?.getString(R.string.favorites),null)
        )
    }

    private fun bindRecycler() {
        favoritesRecyclerView.isNestedScrollingEnabled = true
        favoritesRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        favoritesRecyclerView.adapter = FavoritesListAdapter()
        favoritesRecyclerView.itemAnimator = null

        if (favoritesRecyclerView.itemDecorationCount == 0)
            favoritesRecyclerView.addItemDecoration(
                CeilsDecoration(1
                , resources?.getDimensionPixelSize(R.dimen.ceil_grid_padding))
            )

    }

    override fun onViewBound(v: View) {
        setToolbar()
        bindRecycler()
        detachDisposable.addAll(
            (favoritesRecyclerView.adapter as FavoritesListAdapter)
                .onItemClicked
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    Timber.d("got card clicked $it")
                    router.pushController(RouterTransaction.with(EstateScreenController(it.id)))
                }
        )
    }

    override fun render(viewState: FavoritesScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is FavoritesScreenViewState.Idle -> render(viewState)
            is FavoritesScreenViewState.FavoriteEstatesLoaded -> render(viewState)
            is FavoritesScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: FavoritesScreenViewState.Idle){

    }

    private fun render(viewState: FavoritesScreenViewState.SomeError) =
        showError(viewState.exception)


    private fun render(viewState: FavoritesScreenViewState.FavoriteEstatesLoaded) {
        (favoritesRecyclerView.adapter as FavoritesListAdapter).addItems(viewState.items)
    }

    private fun getComponent() = DaggerFavoritesScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.favorites_screen


}