package com.biganto.visual.roompark.presentation.screen.favorites.util

import android.view.View
import android.widget.TextView
import butterknife.BindView
import butterknife.ButterKnife
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.domain.model.CommonInfoModel
import com.biganto.visual.roompark.domain.model.EstateModel
import com.biganto.visual.roompark.domain.model.typeDoubleString
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonRecyclerAdapter
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonViewHolder
import com.biganto.visual.roompark.util.extensions.setGone

/**
 * Created by Vladislav Bogdashkin on 16.10.2019.
 */

class FavoritesListAdapter : CommonRecyclerAdapter<FavoriteViewHolder,EstateModel>() {
    override val vhKlazz = FavoriteViewHolder::class
    override fun getVhLayout(): Int = R.layout.favorite_card_viewholder
}


class FavoriteViewHolder(itemView: View) : CommonViewHolder<EstateModel>(itemView) {

    @BindView(R.id.object_card_title) lateinit var estateTitle: TextView
    @BindView(R.id.common_info_block) lateinit var commonInfo:View
    @BindView(R.id.start_tour_button) lateinit var startTour:View

    @BindView(R.id.info_ceil_1) lateinit var info1:View
    @BindView(R.id.info_ceil_2) lateinit var info2:View
    @BindView(R.id.info_ceil_3) lateinit var info3:View
    @BindView(R.id.info_ceil_4) lateinit var info4:View
    @BindView(R.id.info_ceil_5) lateinit var info5:View
    @BindView(R.id.info_ceil_6) lateinit var info6:View
    @BindView(R.id.info_ceil_7) lateinit var info7:View
    @BindView(R.id.info_ceil_8) lateinit var info8:View

    init {
        ButterKnife.bind(this, itemView)
    }

    override fun onViewBound(model: EstateModel) {
        estateTitle.text =
            itemView.context.resources?.getString(model.typeDoubleString(),model.number)
        commonInfo.setGone(model.commonInfo == null)
        model.commonInfo?.let {renderCommonInfo(it)}

        startTour.setGone(model.type != "flat")
    }

    private fun renderCommonInfo(info:CommonInfoModel){

        if (info.building == null) info1.visibility = View.GONE
        else { info1.title().text = "Корпус"; info1.text().text = info.building.toString()}

        if (info.section_begin == null) info1.visibility = View.GONE
        else { info2.title().text = "Секция"; info2.text().text = info.section_begin.toString()}

        if (info.floor == null) info1.visibility = View.GONE
        else { info3.title().text = "Этаж"; info3.text().text = info.floor.toString()}

        if (info.area == null) info1.visibility = View.GONE
        else { info4.title().text = "Общая, м²"; info4.text().text = info.area.toString()}

        if (info.price_meter == null) info1.visibility = View.GONE
        else { info5.title().text = "Цена за м²"; info5.text().text = info.price_meter.toRubles()}

        if (info.price == null) info1.visibility = View.GONE
        else { info6.title().text = "Стоимость"; info6.text().text = info.price.toRubles()}

        if (true) info7.visibility = View.GONE
        else { info7.title().text = "вщщ"; info7.text().text = info.building.toString()}

        if (true) info8.visibility = View.GONE
        else { info8.title().text = "щщщщщз"; info8.text().text = info.building.toString()}

    }

    private fun View.title() = this.findViewById<TextView>(R.id.info_ceil_header)
    private fun View.text() = this.findViewById<TextView>(R.id.info_ceil_content)
    private fun Int.toRubles(): String {
        return String.format("%,d \u20BD",this).replace(',',' ')
    }


}