package com.biganto.visual.roompark.presentation.screen.feed_list

import android.view.View
import androidx.appcompat.widget.Toolbar
import androidx.core.os.bundleOf
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.HeaderToolbarModel
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.article.ArticleScreenController
import com.biganto.visual.roompark.presentation.screen.feed_list.util.ArticlesAdapter
import com.biganto.visual.roompark.util.view_utils.grid.CeilsDecoration
import com.bluelinelabs.conductor.RouterTransaction
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val FEED_ALIAS = "DIRECT_FEED_ID_KEY"

class ArticlesScreenController  :
    BigantoBaseController<ArticlesScreenViewState
        , ArticlesScreen
        , ArticlesScreenPresenter>
    , ArticlesScreen {


    constructor()

    constructor(feedAlias:String):super(bundleOf(FEED_ALIAS to feedAlias))

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: ArticlesScreenPresenter

    @BindView(R.id.articles_recycler_view)
    lateinit var articlesRecyclerView: RecyclerView


    private fun setToolbar() {
        toolBar.showAll()
        toolBar.appBar.visibility = Toolbar.VISIBLE

        toolBar.appBar.setExpanded(false, false)
        toolBar.appBar.setLiftable(true)
        toolBar.appBarScrollable(false)
        articlesRecyclerView.isNestedScrollingEnabled = false

        toolBar.setToolbar(
            HeaderToolbarModel(
                true, null
                , resources?.getString(
                    when (args.getString(FEED_ALIAS)) {
                        "news" -> R.string.news_header
                        else -> R.string.blog_header
                    }
                ), true
            )
            , null
        )
    }

    private fun bindRecycler() {
        articlesRecyclerView.isNestedScrollingEnabled = true
        articlesRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        articlesRecyclerView.adapter = ArticlesAdapter()
        articlesRecyclerView.itemAnimator = null

        if (articlesRecyclerView.itemDecorationCount == 0)
            articlesRecyclerView.addItemDecoration(
                CeilsDecoration(1
                , resources?.getDimensionPixelSize(R.dimen.ceil_grid_padding))
            )

        (articlesRecyclerView.adapter as ArticlesAdapter)
            .onItemClicked.subscribe {
            router.pushController(RouterTransaction.with(ArticleScreenController(it.articleId)))
        }
    }

    override fun onViewBound(v: View) {
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: ArticlesScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is ArticlesScreenViewState.Idle -> render(viewState)
            is ArticlesScreenViewState.ArticlesLoaded -> render(viewState)
            is ArticlesScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: ArticlesScreenViewState.Idle){

    }

    private fun render(viewState: ArticlesScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun render(viewState: ArticlesScreenViewState.ArticlesLoaded) {
        (articlesRecyclerView.adapter as ArticlesAdapter).setItems(viewState.items)
    }

    private fun getComponent() = DaggerArticlesScreenComponent.factory()
        .create(args.getString(FEED_ALIAS)!!
            ,RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.feed_direct_screen

    override fun handleBack(): Boolean {
        Timber.d("handle back in class ")
        router.popController(this)
        return true
    }


}