package com.biganto.visual.roompark.presentation.screen.to_flat

import android.view.KeyEvent
import android.view.View
import android.widget.EditText
import android.widget.TextView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.HeaderToolbarModel
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.estate.EstateScreenController
import com.biganto.visual.roompark.presentation.screen.feeds.EstateTabModel
import com.bluelinelabs.conductor.RouterTransaction
import com.bluelinelabs.conductor.changehandler.FadeChangeHandler
import com.google.android.material.button.MaterialButton
import com.google.android.material.tabs.TabLayout
import com.google.android.material.textfield.TextInputLayout
import com.jakewharton.rxbinding3.view.clicks
import com.jakewharton.rxbinding3.view.keys
import com.jakewharton.rxbinding3.widget.afterTextChangeEvents
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class FindFlatScreenController  :
    BigantoBaseController<FindFlatScreenViewState
        , FindFlatScreen
        , FindFlatScreenPresenter>()
    , FindFlatScreen {


    override fun openFlat(): Observable<Int> =
        findFlatButton.clicks()
            .map { 1 }
            .mergeWith(flatNumberEditor.keys{ it.keyCode == KeyEvent.KEYCODE_ENTER }.map { 1 })
            .doOnNext{ flatNumberEditor.hideKeyboard() }
            .observeOn(AndroidSchedulers.mainThread())

    override fun getFlat(): Observable<FlatRequestModel> =
        flatNumberEditor.afterTextChangeEvents()// keys{ it.keyCode == KeyEvent.KEYCODE_ENTER }
            .filter { flatNumberInput.editText?.text?.isNotEmpty()?:false }
            .map {
                FlatRequestModel(
                    estateTabs[flatTabs.selectedTabPosition].building
                    , flatNumberInput.editText?.text.toString().toInt()
                )
            }
            .debounce (300,TimeUnit.MILLISECONDS)
            .observeOn(AndroidSchedulers.mainThread())

    override fun injectDependencies() {
        getComponent()
    }

    private fun setToolbar(){
        toolBar.showAll()
        toolBar.appBar.setExpanded(false,true)
        toolBar.appBar.liftOnScrollTargetViewId = R.id.favorites_cards_recycler_view
        toolBar.appBar.setLiftable(true)
        toolBar.appBar.setLifted(true)
        toolBar.appBarScrollable(false)

        toolBar.setToolbar(
            HeaderToolbarModel(
                false,null
                ,resources?.getString(R.string.flats),null)
        )
    }


    @BindView(R.id.flat_estate_tabs)
    lateinit var flatTabs: TabLayout

    @BindView(R.id.flatNumberInputer)
    lateinit var flatNumberInput: TextInputLayout

    @BindView(R.id.flatNumberEditText)
    lateinit var flatNumberEditor: EditText

    @BindView(R.id.findFlatButton)
    lateinit var findFlatButton: MaterialButton

    @Inject
    override lateinit var injectedPresenter: FindFlatScreenPresenter

    private val estateTabs = arrayListOf(
        EstateTabModel("Д1",1),
        EstateTabModel("Д2",2),
        EstateTabModel("Д3",3)
    )

    override fun onViewBound(v: View) {
        setToolbar()
        estateTabs.forEach {estate ->
            val tab = flatTabs.newTab()
                .setCustomView(R.layout.to_flat_tab_view)
            tab.customView
                ?.let {
                    it.findViewById<TextView>(R.id.tab_title)?.text = estate.title

                    it.findViewById<TextView>(R.id.tab_divider)?.visibility =
                        if (estateTabs.indexOf(estate) == estateTabs.size - 1) View.GONE
                        else View.VISIBLE
                }

            flatTabs.addTab(tab)
        }
    }

    override fun render(viewState: FindFlatScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is FindFlatScreenViewState.Idle -> render(viewState)
            is FindFlatScreenViewState.SomeError -> render(viewState)
            is FindFlatScreenViewState.FlatFounded -> render(viewState)
            is FindFlatScreenViewState.FlatNotFound -> render(viewState)
            is FindFlatScreenViewState.StartFlat -> render(viewState)
        }
    }

    private fun render(viewState: FindFlatScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun render(viewState: FindFlatScreenViewState.Idle){
    }

    private fun render(viewState: FindFlatScreenViewState.FlatFounded){
        findFlatButton.isEnabled = true
        findFlatButton.text = resources?.getString(R.string.flat_ready_to_watch)
    }

    private fun render(viewState: FindFlatScreenViewState.FlatNotFound){
        findFlatButton.isEnabled = false
        viewState.exceptionString.selectHandler(
            {findFlatButton.text = resources?.getString(it)},
            {findFlatButton.text = it}
        )


    }

    private fun render(viewState: FindFlatScreenViewState.StartFlat){
        router.pushController(
            RouterTransaction.with(EstateScreenController(viewState.esateId))
                .pushChangeHandler(FadeChangeHandler())
                .popChangeHandler(FadeChangeHandler())
        )
    }

    private fun getComponent() = DaggerFindFlatScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.find_flat_screen


}